@@ -6,7 +6,7 @@
 # http://www.apache.org/licenses/LICENSE-2.0OA
 #
 # Authors:
-# - Wen Guan, <wen.guan@cern.ch>, 2019 - 2022
+# - Wen Guan, <wen.guan@cern.ch>, 2019 - 2023
 
 
 """
@@ -569,6 +569,10 @@ class Content_update(BASE, ModelBase):
     __tablename__ = 'contents_update'
     content_id = Column(BigInteger().with_variant(Integer, "sqlite"), primary_key=True)
     substatus = Column(EnumWithValue(ContentStatus))
+    request_id = Column(BigInteger().with_variant(Integer, "sqlite"))
+    transform_id = Column(BigInteger().with_variant(Integer, "sqlite"))
+    workload_id = Column(Integer())
+    coll_id = Column(BigInteger().with_variant(Integer, "sqlite"))
 
 
 class Content_ext(BASE, ModelBase):
@@ -732,6 +736,136 @@ class Command(BASE, ModelBase):
                    Index('COMMANDS_TYPE_ST_PR_IDX', 'cmd_type', 'status', 'destination', 'processing_id'))
 
 
+def create_trigger():
+    func = DDL("""
+        SET search_path TO %s;
+        CREATE OR REPLACE FUNCTION update_dep_contents_status()
+        RETURNS TRIGGER AS $$
+        BEGIN
+            UPDATE %s.contents set substatus = old.substatus where %s.contents.content_dep_id = old.content_id;
+            RETURN OLD;
+        END;
+        $$ LANGUAGE PLPGSQL
+    """ % (DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME))
+
+    trigger_ddl = DDL("""
+        SET search_path TO %s;
+        DROP TRIGGER IF EXISTS update_content_dep_status ON %s.contents_update;
+        CREATE TRIGGER update_content_dep_status BEFORE DELETE ON %s.contents_update
+        for each row EXECUTE PROCEDURE update_dep_contents_status();
+    """ % (DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME))
+
+    event.listen(Content_update.__table__, "after_create", func.execute_if(dialect="postgresql"))
+    event.listen(Content_update.__table__, "after_create", trigger_ddl.execute_if(dialect="postgresql"))
+
+
+def delete_trigger():
+    func = DDL("""
+        SET search_path TO %s;
+        DROP FUNCTION IF EXISTS update_dep_contents_status;
+    """ % (DEFAULT_SCHEMA_NAME))
+    trigger_ddl = DDL("""
+        SET search_path TO %s;
+        DROP TRIGGER IF EXISTS update_content_dep_status ON %s.contents_update;
+    """ % (DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME))
+
+    event.listen(Content_update.__table__, "before_drop", func.execute_if(dialect="postgresql"))
+    event.listen(Content_update.__table__, "before_drop", trigger_ddl.execute_if(dialect="postgresql"))
+
+
+def create_func_to_update_contents():
+    func1 = DDL("""
+        SET search_path TO %s;
+        CREATE OR REPLACE FUNCTION update_contents_to_others(request_id_in int, transform_id_in int)
+        RETURNS INTEGER
+        AS $$
+        DECLARE num_rows INTEGER;
+        BEGIN
+            num_rows := 0;
+
+            UPDATE %s.contents set substatus = d.substatus from
+            (select content_id, content_dep_id, substatus from %s.contents where request_id = request_id_in and transform_id = transform_id_in and content_relation_type = 1 and status != 0) d
+            where %s.contents.request_id = request_id_in and %s.contents.substatus != d.substatus and d.content_id = %s.contents.content_dep_id;
+            GET DIAGNOSTICS num_rows = ROW_COUNT;
+            return num_rows;
+        END;
+        $$ LANGUAGE PLPGSQL
+    """ % (DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME,
+           DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME))
+
+    func2 = DDL("""
+        SET search_path TO %s;
+        CREATE OR REPLACE FUNCTION update_contents_from_others(request_id_in int, transform_id_in int)
+        RETURNS INTEGER
+        AS $$
+        DECLARE num_rows INTEGER;
+        BEGIN
+            num_rows := 0;
+
+            UPDATE %s.contents set substatus = d.substatus from
+            (select content_id, content_dep_id, substatus from %s.contents where request_id = request_id_in and content_relation_type = 1 and status != 0) d
+            where %s.contents.request_id = request_id_in and %s.contents.transform_id = transform_id_in and %s.contents.substatus != d.substatus and d.content_id = %s.contents.content_dep_id;
+            GET DIAGNOSTICS num_rows = ROW_COUNT;
+            return num_rows;
+        END;
+        $$ LANGUAGE PLPGSQL
+    """ % (DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME,
+           DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME))
+
+    event.listen(Content.__table__, "after_create", func1.execute_if(dialect="postgresql"))
+    event.listen(Content.__table__, "after_create", func2.execute_if(dialect="postgresql"))
+
+
+def drop_func_to_update_contents():
+    func = DDL("""
+        SET search_path TO %s;
+        DROP FUNCTION IF EXISTS update_contents_to_others;
+        DROP FUNCTION IF EXISTS update_contents_from_others;
+    """ % (DEFAULT_SCHEMA_NAME))
+    event.listen(Content.__table__, "before_drop", func.execute_if(dialect="postgresql"))
+
+
+def create_proc_to_update_contents():
+    func1 = DDL("""
+        SET search_path TO %s;
+        CREATE OR REPLACE PROCEDURE update_contents_to_others(request_id_in int, transform_id_in int)
+        AS $$
+        BEGIN
+            UPDATE %s.contents set substatus = d.substatus from
+            (select content_id, content_dep_id, substatus from %s.contents where request_id = request_id_in and transform_id = transform_id_in and content_relation_type = 1 and status != 0) d
+            where %s.contents.request_id = request_id_in and %s.contents.substatus != d.substatus and d.content_id = %s.contents.content_dep_id;
+        END;
+        $$ LANGUAGE PLPGSQL
+    """ % (DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME,
+           DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME))
+
+    func2 = DDL("""
+        SET search_path TO %s;
+        CREATE OR REPLACE PROCEDURE update_contents_from_others(request_id_in int, transform_id_in int)
+        AS $$
+        BEGIN
+
+            UPDATE %s.contents set substatus = d.substatus from
+            (select content_id, content_dep_id, substatus from %s.contents where request_id = request_id_in and content_relation_type = 1 and status != 0) d
+            where %s.contents.request_id = request_id_in and %s.contents.transform_id = transform_id_in and %s.contents.substatus != d.substatus and d.content_id = %s.contents.content_dep_id;
+        END;
+        $$ LANGUAGE PLPGSQL
+    """ % (DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME,
+           DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME, DEFAULT_SCHEMA_NAME))
+
+    event.listen(Content.__table__, "after_create", func1.execute_if(dialect="postgresql"))
+    event.listen(Content.__table__, "after_create", func2.execute_if(dialect="postgresql"))
+
+
+def drop_proc_to_update_contents():
+    func = DDL("""
+        SET search_path TO %s;
+        DROP PROCEDURE IF EXISTS update_contents_to_others;
+        DROP PROCEDURE IF EXISTS update_contents_from_others;
+    """ % (DEFAULT_SCHEMA_NAME))
+    event.listen(Content.__table__, "before_drop", func.execute_if(dialect="postgresql"))
+
+
 def register_models(engine):
     """
     Creates database tables for all models with the given engine
@@ -740,6 +874,8 @@ def register_models(engine):
     # models = (Request, Workprogress, Transform, Workprogress2transform, Processing, Collection, Content, Health, Message)
     models = (Request, Transform, Processing, Collection, Content, Content_update, Content_ext, Health, Message, Command)
 
+    create_proc_to_update_contents()
+
     for model in models:
         # if not engine.has_table(model.__tablename__, model.metadata.schema):
         model.metadata.create_all(engine)   # pylint: disable=maybe-no-member
@@ -753,16 +889,7 @@ def unregister_models(engine):
     # models = (Request, Workprogress, Transform, Workprogress2transform, Processing, Collection, Content, Health, Message)
     models = (Request, Transform, Processing, Collection, Content, Content_update, Content_ext, Health, Message, Command)
 
+    drop_proc_to_update_contents()
+
     for model in models:
         model.metadata.drop_all(engine)   # pylint: disable=maybe-no-member
-
-
-@event.listens_for(Content_update.__table__, "after_create")
-def _update_content_dep_status(target, connection, **kw):
-    DDL("""
-        CREATE TRIGGER update_content_dep_status BEFORE DELETE ON contents_update
-        for each row
-        BEGIN
-            UPDATE contents set substatus = :old.substatus where contents.content_dep_id = :old.content_id;
-        END;
-    """)
