@@ -172,6 +172,32 @@ def get_request_ids_by_workload_id(workload_id, session=None):
         raise exceptions.NoObject('request with workload_id:%s cannot be found: %s' % (workload_id, error))
 
 
+@read_session
+def get_request_ids_by_name(name, session=None):
+    """
+    Get request ids or raise a NoObject exception.
+
+    :param name: name of the request.
+    :param session: The database session in use.
+
+    :raises NoObject: If no request is founded.
+
+    :returns: Request {name:id} dict.
+    """
+    try:
+        query = session.query(models.Request.request_id, models.Request.name)\
+                       .with_hint(models.Request, "INDEX(REQUESTS REQUESTS_SCOPE_NAME_IDX)", "oracle")\
+                       .filter(models.Request.name.like(name.replace('*', '%')))
+        tmp = query.all()
+        ret_ids = {}
+        if tmp:
+            for req in tmp:
+                ret_ids[req[1]] = req[0]
+        return ret_ids
+    except sqlalchemy.orm.exc.NoResultFound as error:
+        raise exceptions.NoObject('request with name:%s cannot be found: %s' % (name, error))
+
+
 @read_session
 def get_request_ids(request_id=None, workload_id=None, session=None):
     """
@@ -418,14 +444,23 @@ def get_requests(request_id=None, workload_id=None, with_detail=False, with_meta
                     # t2 = dict(t)
                     t2 = dict(zip(t.keys(), t))
 
-                    if 'request_metadata' in t2 and t2['request_metadata'] and 'workflow' in t2['request_metadata']:
-                        workflow = t2['request_metadata']['workflow']
-                        workflow_data = None
-                        if 'processing_metadata' in t2 and t2['processing_metadata'] and 'workflow_data' in t2['processing_metadata']:
-                            workflow_data = t2['processing_metadata']['workflow_data']
-                        if workflow is not None and workflow_data is not None:
-                            workflow.metadata = workflow_data
-                            t2['request_metadata']['workflow'] = workflow
+                    if 'request_metadata' in t2 and t2['request_metadata']:
+                        if 'workflow' in t2['request_metadata']:
+                            workflow = t2['request_metadata']['workflow']
+                            workflow_data = None
+                            if 'processing_metadata' in t2 and t2['processing_metadata'] and 'workflow_data' in t2['processing_metadata']:
+                                workflow_data = t2['processing_metadata']['workflow_data']
+                            if workflow is not None and workflow_data is not None:
+                                workflow.metadata = workflow_data
+                                t2['request_metadata']['workflow'] = workflow
+                        if 'build_workflow' in t2['request_metadata']:
+                            build_workflow = t2['request_metadata']['build_workflow']
+                            build_workflow_data = None
+                            if 'processing_metadata' in t2 and t2['processing_metadata'] and 'build_workflow_data' in t2['processing_metadata']:
+                                build_workflow_data = t2['processing_metadata']['build_workflow_data']
+                            if build_workflow is not None and build_workflow_data is not None:
+                                build_workflow.metadata = build_workflow_data
+                                t2['request_metadata']['build_workflow'] = build_workflow
 
                     rets.append(t2)
             return rets
@@ -485,14 +520,23 @@ def get_requests(request_id=None, workload_id=None, with_detail=False, with_meta
                     # t2 = dict(t)
                     t2 = dict(zip(t.keys(), t))
 
-                    if 'request_metadata' in t2 and t2['request_metadata'] and 'workflow' in t2['request_metadata']:
-                        workflow = t2['request_metadata']['workflow']
-                        workflow_data = None
-                        if 'processing_metadata' in t2 and t2['processing_metadata'] and 'workflow_data' in t2['processing_metadata']:
-                            workflow_data = t2['processing_metadata']['workflow_data']
-                        if workflow is not None and workflow_data is not None:
-                            workflow.metadata = workflow_data
-                            t2['request_metadata']['workflow'] = workflow
+                    if 'request_metadata' in t2 and t2['request_metadata']:
+                        if 'workflow' in t2['request_metadata']:
+                            workflow = t2['request_metadata']['workflow']
+                            workflow_data = None
+                            if 'processing_metadata' in t2 and t2['processing_metadata'] and 'workflow_data' in t2['processing_metadata']:
+                                workflow_data = t2['processing_metadata']['workflow_data']
+                            if workflow is not None and workflow_data is not None:
+                                workflow.metadata = workflow_data
+                                t2['request_metadata']['workflow'] = workflow
+                        if 'build_workflow' in t2['request_metadata']:
+                            build_workflow = t2['request_metadata']['build_workflow']
+                            build_workflow_data = None
+                            if 'processing_metadata' in t2 and t2['processing_metadata'] and 'build_workflow_data' in t2['processing_metadata']:
+                                build_workflow_data = t2['processing_metadata']['build_workflow_data']
+                            if build_workflow is not None and build_workflow_data is not None:
+                                build_workflow.metadata = build_workflow_data
+                                t2['request_metadata']['build_workflow'] = build_workflow
 
                     rets.append(t2)
             return rets
@@ -631,14 +675,23 @@ def get_requests(request_id=None, workload_id=None, with_detail=False, with_meta
                     # t2 = dict(t)
                     t2 = dict(zip(t.keys(), t))
 
-                    if 'request_metadata' in t2 and t2['request_metadata'] and 'workflow' in t2['request_metadata']:
-                        workflow = t2['request_metadata']['workflow']
-                        workflow_data = None
-                        if 'processing_metadata' in t2 and t2['processing_metadata'] and 'workflow_data' in t2['processing_metadata']:
-                            workflow_data = t2['processing_metadata']['workflow_data']
-                        if workflow is not None and workflow_data is not None:
-                            workflow.metadata = workflow_data
-                            t2['request_metadata']['workflow'] = workflow
+                    if 'request_metadata' in t2 and t2['request_metadata']:
+                        if 'workflow' in t2['request_metadata']:
+                            workflow = t2['request_metadata']['workflow']
+                            workflow_data = None
+                            if 'processing_metadata' in t2 and t2['processing_metadata'] and 'workflow_data' in t2['processing_metadata']:
+                                workflow_data = t2['processing_metadata']['workflow_data']
+                            if workflow is not None and workflow_data is not None:
+                                workflow.metadata = workflow_data
+                                t2['request_metadata']['workflow'] = workflow
+                        if 'build_workflow' in t2['request_metadata']:
+                            build_workflow = t2['request_metadata']['build_workflow']
+                            build_workflow_data = None
+                            if 'processing_metadata' in t2 and t2['processing_metadata'] and 'build_workflow_data' in t2['processing_metadata']:
+                                build_workflow_data = t2['processing_metadata']['build_workflow_data']
+                            if build_workflow is not None and build_workflow_data is not None:
+                                build_workflow.metadata = build_workflow_data
+                                t2['request_metadata']['build_workflow'] = build_workflow
 
                     rets.append(t2)
             return rets
@@ -806,7 +859,7 @@ def get_requests_by_status_type(status, request_type=None, time_period=None, req
 
 
 @transactional_session
-def update_request(request_id, parameters, session=None):
+def update_request(request_id, parameters, update_request_metadata=False, session=None):
     """
     update an request.
 
@@ -826,17 +879,30 @@ def update_request(request_id, parameters, session=None):
         if 'update_poll_period' in parameters and type(parameters['update_poll_period']) not in [datetime.timedelta]:
             parameters['update_poll_period'] = datetime.timedelta(seconds=parameters['update_poll_period'])
 
-        if 'request_metadata' in parameters and 'workflow' in parameters['request_metadata']:
-            workflow = parameters['request_metadata']['workflow']
+        if 'request_metadata' in parameters:
+            if 'workflow' in parameters['request_metadata']:
+                workflow = parameters['request_metadata']['workflow']
 
-            if workflow is not None:
-                workflow.refresh_works()
-                if 'processing_metadata' not in parameters or not parameters['processing_metadata']:
-                    parameters['processing_metadata'] = {}
-                parameters['processing_metadata']['workflow_data'] = workflow.metadata
+                if workflow is not None:
+                    workflow.refresh_works()
+                    if 'processing_metadata' not in parameters or not parameters['processing_metadata']:
+                        parameters['processing_metadata'] = {}
+                    parameters['processing_metadata']['workflow_data'] = workflow.metadata
+            if 'build_workflow' in parameters['request_metadata']:
+                build_workflow = parameters['request_metadata']['build_workflow']
+
+                if build_workflow is not None:
+                    build_workflow.refresh_works()
+                    if 'processing_metadata' not in parameters or not parameters['processing_metadata']:
+                        parameters['processing_metadata'] = {}
+                    parameters['processing_metadata']['build_workflow_data'] = build_workflow.metadata
 
         if 'request_metadata' in parameters:
-            del parameters['request_metadata']
+            if not update_request_metadata:
+                del parameters['request_metadata']
+            else:
+                parameters['_request_metadata'] = parameters['request_metadata']
+                del parameters['request_metadata']
         if 'processing_metadata' in parameters:
             parameters['_processing_metadata'] = parameters['processing_metadata']
             del parameters['processing_metadata']
